from image_to_table.animation import animate, clear_animation
from image_to_table.generate import generate_timetable
from image_to_table.json_extractor import extract, save_json_to_file
from image_to_table.visualiser import json_to_table


import os
import random
import time

# Define the symbols for animation
icons = ["  ", "🟤", "👻"]

def generate_wave_frames(term_size):
    frames = []
    waves = []
    bg = [0] * term_size
    i = 10
    iend = term_size - 1
    
    while iend > 0:
        line = list(bg)
        
        if i % 10 == 0:
            waves.append(0)
        
        for iw in range(len(waves)):
            line[waves[iw]] = 1
            if waves[iw] >= iend:
                continue
            waves[iw] += 1
            if waves[iw] == iend:
                iend -= 1
        
        frames.append(line)
        i += 1
    
    return frames

def generate_surf_frames(term_size):
    frames = []
    bg = [1] * term_size
    rider = random.randint(2, len(icons) - 1)
    
    for i in reversed(range(term_size)):
        line = list(bg)
        line[i] = rider
        frames.append(line)
    
    frames.append(bg)
    return frames

def display_frames(frames, icons):
    for line in frames:
        print("".join(map(lambda x: icons[x], line)), end="\r", flush=True)
        time.sleep(0.1)

def clear_animation(term_size):
    # Clear the terminal by printing empty lines
    for _ in range(term_size):
        print()
        
def animate():
    term_size = os.get_terminal_size().columns //2
    
    while True:
        frames = generate_surf_frames(term_size)
        display_frames(frames, icons)
        time.sleep(1)



import re

def extract(input_text: str) -> str:
    """
    Extracts JSON content from a given input text.

    Args:
        input_text (str): The input text containing JSON content.

    Returns:
        str: The extracted JSON content, or None if no JSON content is found.
    """
    pattern = r"```json(.*?)```"

    match = re.search(pattern, input_text, re.DOTALL)
    
    if (match):
        return match.group(1).strip()
    else:
        raise("No JSON content found in the input text.")

def save_json_to_file(json_content: str, file_path: str) -> None:
    """
    Saves the given JSON content to a file.

    Args:
        json_content (str): The JSON content to save.
        file_path (str): The path to the file to save the JSON content to.
    """
    try:
        with open(file_path, "x") as file:
            file.write(json_content)
    except Exception as e:
        print(f"Error saving the content as json: {e}")


import json
from tabulate import tabulate

def json_to_table(file_path: str, number_char_cell: int = 18) -> None:
    
    output = []
    
    try:
        with open(file_path, "r") as file:
            json_data = file.read()
    except Exception as e:
        print(f"Error saving the content as json: {e}")
        return
        
    json_str = json.loads(json_data)
    
    first_row = ["hours"]
    first_row.extend(list(json_str.keys()))
    
    first_col = [[key] for key in json_str[first_row[1]]]
    
    output.append(first_row)
    output.extend(first_col)
    
    for i in range(1,len(output)):
        row = output[i]
        time = row[0]
        
        for key in json_str.keys():
            appoint = json_str[key][time]
            value = list(appoint.values())
            if value != []:
                row.append(value[0]['course'][:number_char_cell])
            else:
                row.append("-X-")
                
    print(tabulate(output,headers="firstrow",tablefmt="grid"))
    


from llama_index.multi_modal_llms.gemini import GeminiMultiModal
from llama_index.core.multi_modal_llms.generic_utils import load_image_urls
from llama_index.core import SimpleDirectoryReader
from image_to_table.json_extractor import extract, save_json_to_file
import time
from threading import Thread
from image_to_table.animation import animate, clear_animation

PROMPT = """you are an ai that returns a time table in json format and in english from a given image
this is an example of how the output should look like:
```json	
{
  "Monday": {
    "08:30-10:00": {
      "IA1.1": {
        "course": "Reseaux informatiques (TP)",
        "instructor": "BEN FARAH Ahmed",
        "location": "I12"
      },
      "IA1.2": {
        "course": "Architecture des microprocesseurs et des multiprocesseurs (TP)",
        "instructor": "KHADRAOUI Imen",
        "location": "I13"
      }
    },
    "10:15-11:45": {
      "IA1.1": {
        "course": "Reseaux informatiques (TP)",
        "instructor": "BEN FARAH Ahmed",
        "location": "I12"
      },
      "IA1.2": {
        "course": "Architecture des microprocesseurs et des multiprocesseurs (TP)",
        "instructor": "KHADRAOUI Imen",
        "location": "I13"
      }
    },
    "12:00-13:00": {},
    "13:00-14:30": {
      "IA1.1": {
        "course": "Réseaux de Petri",
        "instructor": "ABDELLAOUI Mehrez",
        "location": "B02"
      }
    },
    "14:45-16:15": {
      "IA1.1": {
        "course": "Elements de puissance (1/15)",
        "instructor": "BEL HADJ IBRAHIM Anis",
        "location": "R03"
      }
    },
    "16:30-18:00": {}
  },
  "Tuesday": {
    "08:30-10:00": {
      "IA1.1": {
        "course": "Automatique lineaire echantillonne (TP)",
        "instructor": "BEMBILI Sana",
        "location": "E13"
      }
    },
    "10:15-11:45": {
      "IA1.1": {
        "course": "Automatique lineaire echantillonne (TP)",
        "instructor": "BEMBILI Sana",
        "location": "E13"
      }
    },
    "12:00-13:00": {},
    "13:00-14:30": {
      "IA1.2": {
        "course": "Programmation Orientee Objet",
        "instructor": "ABDELATTIF Takoua",
        "location": "B04"
      }
    },
    "14:45-16:15": {
      "IA1.2": {
        "course": "Elements de puissance",
        "instructor": "BEL HADJ IBRAHIM Anis",
        "location": "B04"
      }
    },
    "16:30-18:00": {}
  },
  "Wednesday": {
    "08:30-10:00": {
      "IA1.1": {
        "course": "Reseaux informatiques (1/15)",
        "instructor": "BEN ARBIA Anis",
        "location": ""
      }
    },
    "10:15-11:45": {
      "IA1.1": {
        "course": "Algorithmique et Structures de donnees II (1/15)",
        "instructor": "CHAINBI Walid",
        "location": "B04"
      }
    },
    "12:00-13:00": {},
    "13:00-14:30": {},
    "14:45-16:15": {},
    "16:30-18:00": {}
  },
  "Thursday": {
    "08:30-10:00": {
      "IA1.1": {
        "course": "Algorithmique et Structures de donnees II (1/15)",
        "instructor": "CHAINBI Walid",
        "location": "B04"
      }
    },
    "10:15-11:45": {
      "IA1.1": {
        "course": "Reseaux informatiques",
        "instructor": "BEN ARBIA Anis",
        "location": "R03"
      }
    },
    "12:00-13:00": {},
    "13:00-14:30": {
      "IA1.1": {
        "course": "Circuits Programmables (FPGA)",
        "instructor": "BOUZOUITA Badreddine",
        "location": "B11"
      }
    },
    "14:45-16:15": {
      "IA1.2": {
        "course": "Architecture des microprocesseurs et des\nmultiprocesseurs (1/15)",
        "instructor": "BRAHMI Nabiha",
        "location": "B11"
      }
    },
    "16:30-18:00": {}
  },
  "Friday": {
    "08:30-10:00": {
      "IA1.1": {
        "course": "Anglais II",
        "instructor": "RDIFI Eya",
        "location": "B12 (Labo\nAnglais)"
      },
      "IA1.2": {
        "course": "GTE1, IA1,\nIA2,\nProfessional\nCareer\nDevelopment",
        "instructor": "AYARA\nKhawla\nAmphi",
        "location": ""
      }
    },
    "10:15-11:45": {
      "IA1.1": {
        "course": "Circuits\nProgrammables\n(FPGA) (TP)",
        "instructor": "SOUILEM Haifa",
        "location": "E21"
      },
      "IA1.2": {
        "course": "Francais",
        "instructor": "CHEHIDI Latifa",
        "location": "A04 (Labo\nFrancais)"
      }
    },
    "12:00-13:00": {},
    "13:00-14:30": {
      "IA1.1": {
        "course": "Circuits\nProgrammables\n(FPGA) (TP)",
        "instructor": "SOUILEM Haifa",
        "location": "E21"
      }
    },
    "14:45-16:15": {
      "IA1.1": {
        "course": "Automatique\nlineaire\nechantillonne",
        "instructor": "SAAD Ihssen",
        "location": "R04"
      }
    },
    "16:30-18:00": {
      "IA1.1": {
        "course": "Probabilites et\nstatistiques",
        "instructor": "GHOZZI\nAouicha",
        "location": "R01"
      }
    }
  },
  "Saturday": {
    "08:30-10:00": {
      "IA1.1": {
        "course": "Anglais II",
        "instructor": "MABROUK\nRaouia",
        "location": "B13 (Labo\nAnglais)"
      },
      "IA1.2": {
        "course": "Programmation\nOrientee Objet\n(TP)",
        "instructor": "TRABELSI Salma",
        "location": "I21"
      }
    },
    "10:15-11:45": {
      "IA1.2": {
        "course": "Programmation\nOrientee Objet\n(TP)",
        "instructor": "TRABELSI Salma",
        "location": "I21"
      }
    },
    "12:00-13:00": {},
    "13:00-14:30": {},
    "14:45-16:15": {},
    "16:30-18:00": {}
  }
}
```	
"""

def generate_timetable(PATH:str,API_KEY:str)->None:
    """
    Generates a timetable in JSON format based on the provided prompt and image in ./images directory.

    Returns:
        None
    """
    
    """
    try:
      with Image.open("images/test.png") as img:
        img.show()
    except Exception as e:
      print(f"Error opening image: {e}")"""
      
    Thread(target=animate, daemon=True).start()
    
    # load image documents from local directory
    image_documents = SimpleDirectoryReader("images").load_data()

    mm_llm = GeminiMultiModal(model_name="models/gemini-1.5-pro", api_key=API_KEY)
    start = time.time()
    response = mm_llm.complete(
        prompt=PROMPT, image_documents=image_documents
    ).text
    end = time.time()

    print("Time taken: ", int(end-start),"sec \n","\n-------------------------\n" ,"Table generated!!")

    try:
        extracted_json = extract(response)
        save_json_to_file(extracted_json, PATH)
        print("\n-------------------------\nOutput saved to table.json!!")
    except Exception as e:
        print(f"Error extracting JSON content: {e}")

    


from setuptools import setup, find_packages

VERSION = '1.0.2'
DESCRIPTION = 'Extracting timetable from image and converting it to JSON format.'
LONG_DESCRIPTION = 'A package that allows to extract timetables from images and converting them to JSON format using Gemini MM-LLM.'

# Setting up
setup(
    name="Image-to-Table",
    version=VERSION,
    author="Adem Bouatay",
    author_email="<adem.bouatay@eniso.u-sousse.tn>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(include=['image_to_table', 'image_to_table.*']),
    install_requires=['llama-index', 'llama-index-multi-modal-llms-gemini', 'pillow', 'tabulate'],
    keywords=['python', 'image', 'json', 'MM LLM', 'AI'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
